{ Author:  
  Summary:      }

unit MUI;

interface

uses
  Classes, Controls, Forms, MuiDict;

type
  TOnComponentTranslate = procedure(Sender: TObject; const Component: TComponent; var Process: boolean) of object;
  { Sender: RTFM VCL Manual;
    Component:  ;
    Process: ,        ;}
  TTranslateBase = (tbName, tbCaption);

type
  TMUI = class(TComponent) //   
  private
    FTranslateBase: TTranslateBase;
    FParent: TForm; // . 
    FParentOnCreate, FParentOnDestroy: TNotifyEvent; // . 
    FIgnoreTag: integer; // . 
    FDict: TDict; // . 
    FAutoProcess: boolean; // . 
    FOnComponentTranslate: TOnComponentTranslate; // . 
  protected
    procedure   SetParent(const Value: TForm); // . 
    procedure   ParentCreate(Sender: TObject); // - OnCreate  Paent
    procedure   ParentDestroy(Sender: TObject); // - OnDestroy  Paent
    procedure   SetDict(const Value: TDict); // . 
  public
    constructor Create(AOwner: TComponent); overload; override; // RTFM Object Pascal Reference
    constructor Create(AOwner: TComponent; const Dict: TDict); reintroduce; overload; // RTFM Object Pascal Reference
    constructor Create(AOwner: TComponent; const Dict: TDict; const OwnerAutoProcess: boolean); reintroduce; overload; // RTFM Object Pascal Reference
    destructor  Destroy; override; // RTFM Object Pascal Reference
    procedure   ProcessForm(const Form: TForm); //      
    procedure   ProcessParent; //   Parent
    function    Translate(const Text: string; const DefText: string = ''): string;
                              { 1.     Dictionary
                                2.   ,  Dictionary.OnUnknownText()
                                3.       
                                4.    }

  published
    property    IgnoreTag: integer read FIgnoreTag write FIgnoreTag default 0; //       (0 -  )
    property    ParentForm: TForm read FParent write SetParent default nil; // ,    
    property    Dictionary: TDict read FDict write SetDict default nil; //       (  !)
    property    AutoProcess: boolean read FAutoProcess write FAutoProcess default true; //         
    property    TranslateBase: TTranslateBase read FTranslateBase write FTranslateBase default tbCaption;

    property    OnComponentTranslate: TOnComponentTranslate read FOnComponentTranslate write FOnComponentTranslate default nil; //    .
  end;

implementation

{$R MUI.dcr}

uses //   TControl  TWinControl    
  SysUtils, StdCtrls, ExtCtrls, Buttons, {Menus,} Dialogs, {CheckLst,} ComCtrls{,
  Valedit, Mask, Grids, Chart};

constructor TMUI.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FParent:= nil;
  FParentOnCreate:= nil;
  FParentOnDestroy:= nil;
  FIgnoreTag:= 0;
  FDict:= nil;
  FOnComponentTranslate:= nil;
  FAutoProcess:= true;
  FTranslateBase:= tbCaption;
end;

constructor TMUI.Create(AOwner: TComponent; const Dict: TDict);
begin
  Create(AOwner);
  FDict:= Dict;
end;

constructor TMUI.Create(AOwner: TComponent; const Dict: TDict; const OwnerAutoProcess: boolean);
begin
  Create(AOwner);
  FDict:= Dict;
  if OwnerAutoProcess and (AOwner is TForm) then ProcessForm(AOwner as TForm);
end;

destructor TMUI.Destroy;
begin
  SetParent(nil);
  inherited Destroy;
end;

procedure TMUI.SetParent(const Value: TForm);
begin
  if Value <> FParent then
    begin
      if Assigned(FParent) and (not (csDesigning in Self.ComponentState)) then
        try
          FParent.OnCreate:= FParentOnCreate;
          FParent.OnDestroy:= FParentOnDestroy;
        except
        end;

      FParent:= Value;
      if Assigned(FParent) and (not (csDesigning in Self.ComponentState)) then //   RunTime
        begin
          FParentOnCreate:= FParent.OnCreate;
          FParent.OnCreate:= ParentCreate;
          FParentOnDestroy:= FParent.OnDestroy;
          FParent.OnDestroy:= ParentDestroy;
        end;
    end;
end;

procedure TMUI.ParentCreate(Sender: TObject);
begin
  if FAutoProcess then ProcessParent;
  if Assigned(FParentOnCreate) then FParentOnCreate(FParent);
end;

procedure TMUI.ParentDestroy(Sender: TObject);
begin
  try
    if Assigned(FParentOnDestroy) then FParentOnDestroy(FParent);
  finally
    FParent:= nil;
    FParentOnCreate:= nil;
    FParentOnDestroy:= nil;
  end;
end;

procedure TMUI.SetDict(const Value: TDict);
begin
  if Value <> FDict then
    begin
      FDict:= Value;
//      if (FAutoProcess) and (not (csDesigning in Self.ComponentState)) then ProcessParent;
    end;
end;

function TMUI.Translate(const Text: string; const DefText: string = ''): string;
begin
  if Assigned(FDict) then
    Result:= FDict.Translate(Text, DefText)
  else
    if DefText = '' then
      Result:= Text
    else
      Result:= DefText;
end;

procedure TMUI.ProcessForm(const Form: TForm);
var
  i, j, k: integer;
  Control: TComponent;
  Process: boolean;
begin
  if Assigned(Form) then
    begin
      Process:= true;
      if Assigned(FOnComponentTranslate) then FOnComponentTranslate(Self, Form, Process);
      if Process then
        if FTranslateBase = tbCaption then
          begin
            Form.Caption:= Translate(Form.Caption);
            Form.Hint:= Translate(Form.Hint);
          end
        else
          begin
            Form.Caption:= Translate(Form.Name);
            Form.Hint:= Translate(Form.Name+'.hint');
          end;

      for j:= 0 to Form.ComponentCount-1 do
        if (FIgnoreTag = 0) or (Form.Components[j].Tag <> FIgnoreTag) then
          begin
            Control:= Form.Components[j];
            Process:= true;
            if Assigned(FOnComponentTranslate) then FOnComponentTranslate(Self, Control, Process);
            if Process then
              begin
                // Visual components
                if Control is TButton then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TButton).Caption:= Translate((Control as TButton).Caption);
                      (Control as TButton).Hint:= Translate((Control as TButton).Hint);
                    end
                  else
                    begin
                      (Control as TButton).Caption:= Translate((Control as TButton).Name, (Control as TButton).Caption);
                      (Control as TButton).Hint:= Translate((Control as TButton).Name+'.hint', (Control as TButton).Hint);
                    end;
                if Control is TLabel then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TLabel).Caption:= Translate((Control as TLabel).Caption);
                      (Control as TLabel).Hint:= Translate((Control as TLabel).Hint);
                    end
                  else
                    begin
                      (Control as TLabel).Caption:= Translate((Control as TLabel).Name, (Control as TLabel).Caption);
                      (Control as TLabel).Hint:= Translate((Control as TLabel).Name+'.hint', (Control as TLabel).Hint);
                    end;
                {if Control is TMenuItem then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TMenuItem).Caption:= Translate((Control as TMenuItem).Caption);
                      (Control as TMenuItem).Hint:= Translate((Control as TMenuItem).Hint);
                    end
                  else
                    begin
                      (Control as TMenuItem).Caption:= Translate((Control as TMenuItem).Name, (Control as TMenuItem).Caption);
                      (Control as TMenuItem).Hint:= Translate((Control as TMenuItem).Name+'.hint', (Control as TMenuItem).Hint);
                    end;}
                if Control is TCheckBox then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TCheckBox).Caption:= Translate((Control as TCheckBox).Caption);
                      (Control as TCheckBox).Hint:= Translate((Control as TCheckBox).Hint);
                    end
                  else
                    begin
                      (Control as TCheckBox).Caption:= Translate((Control as TCheckBox).Name, (Control as TCheckBox).Caption);
                      (Control as TCheckBox).Hint:= Translate((Control as TCheckBox).Name+'.hint', (Control as TCheckBox).Hint);
                    end;
                {if Control is TRadioButton then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TRadioButton).Caption:= Translate((Control as TRadioButton).Caption);
                      (Control as TRadioButton).Hint:= Translate((Control as TRadioButton).Hint);
                    end
                  else
                    begin
                      (Control as TRadioButton).Caption:= Translate((Control as TRadioButton).Name, (Control as TRadioButton).Caption);
                      (Control as TRadioButton).Hint:= Translate((Control as TRadioButton).Name+'.hint', (Control as TRadioButton).Hint);
                    end;
                if Control is TBitBtn then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TBitBtn).Caption:= Translate((Control as TBitBtn).Caption);
                      (Control as TBitBtn).Hint:= Translate((Control as TBitBtn).Hint);
                    end
                  else
                    begin
                      (Control as TBitBtn).Caption:= Translate((Control as TBitBtn).Name, (Control as TBitBtn).Caption);
                      (Control as TBitBtn).Hint:= Translate((Control as TBitBtn).Name+'.hint', (Control as TBitBtn).Hint);
                    end;}
                if Control is TSpeedButton then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TSpeedButton).Caption:= Translate((Control as TSpeedButton).Caption);
                      (Control as TSpeedButton).Hint:= Translate((Control as TSpeedButton).Hint);
                    end
                  else
                    begin
                      (Control as TSpeedButton).Caption:= Translate((Control as TSpeedButton).Name, (Control as TSpeedButton).Caption);
                      (Control as TSpeedButton).Hint:= Translate((Control as TSpeedButton).Name+'.hint', (Control as TSpeedButton).Hint);
                    end;
                {if Control is TStaticText then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TStaticText).Caption:= Translate((Control as TStaticText).Caption);
                      (Control as TStaticText).Hint:= Translate((Control as TStaticText).Hint);
                    end
                  else
                    begin
                      (Control as TStaticText).Caption:= Translate((Control as TStaticText).Name, (Control as TStaticText).Caption);
                      (Control as TStaticText).Hint:= Translate((Control as TStaticText).Name+'.hint', (Control as TStaticText).Hint);
                    end;
                if Control is TLabeledEdit then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TLabeledEdit).EditLabel.Caption:= Translate((Control as TLabeledEdit).EditLabel.Caption);
                      (Control as TLabeledEdit).Hint:= Translate((Control as TLabeledEdit).Hint);
                    end
                  else
                    begin
                      (Control as TLabeledEdit).EditLabel.Caption:= Translate((Control as TLabeledEdit).Name, (Control as TLabeledEdit).EditLabel.Caption);
                      (Control as TLabeledEdit).Hint:= Translate((Control as TLabeledEdit).Name+'.hint', (Control as TLabeledEdit).Hint);
                    end;}

                // Hint-only visual components
                {if Control is TImage then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TImage).Hint:= Translate((Control as TImage).Hint);
                    end
                  else
                    begin
                      (Control as TImage).Hint:= Translate((Control as TImage).Name+'.hint', (Control as TImage).Hint);
                    end;
                if Control is TEdit then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TEdit).Hint:= Translate((Control as TEdit).Hint);
                    end
                  else
                    begin
                      (Control as TEdit).Hint:= Translate((Control as TEdit).Name+'.hint', (Control as TEdit).Hint);
                    end;}
                if Control is TMemo then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TMemo).Hint:= Translate((Control as TMemo).Hint);
                    end
                  else
                    begin
                      (Control as TMemo).Hint:= Translate((Control as TMemo).Name+'.hint', (Control as TMemo).Hint);
                    end;
                {if Control is TScrollBar then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TScrollBar).Hint:= Translate((Control as TScrollBar).Hint);
                    end
                  else
                    begin
                      (Control as TScrollBar).Hint:= Translate((Control as TScrollBar).Name+'.hint', (Control as TScrollBar).Hint);
                    end;}
                {if Control is TMaskEdit then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TMaskEdit).Hint:= Translate((Control as TMaskEdit).Hint);
                    end
                  else
                    begin
                      (Control as TMaskEdit).Hint:= Translate((Control as TMaskEdit).Name+'.hint', (Control as TMaskEdit).Hint);
                    end;
                if Control is TStringGrid then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TStringGrid).Hint:= Translate((Control as TStringGrid).Hint);
                    end
                  else
                    begin
                      (Control as TStringGrid).Hint:= Translate((Control as TStringGrid).Name+'.hint', (Control as TStringGrid).Hint);
                    end;
                if Control is TDrawGrid then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TDrawGrid).Hint:= Translate((Control as TDrawGrid).Hint);
                    end
                  else
                    begin
                      (Control as TDrawGrid).Hint:= Translate((Control as TDrawGrid).Name+'.hint', (Control as TDrawGrid).Hint);
                    end;}
                {if Control is TShape then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TShape).Hint:= Translate((Control as TShape).Hint);
                    end
                  else
                    begin
                      (Control as TShape).Hint:= Translate((Control as TShape).Name+'.hint', (Control as TShape).Hint);
                    end;
                if Control is TBevel then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TBevel).Hint:= Translate((Control as TBevel).Hint);
                    end
                  else
                    begin
                      (Control as TBevel).Hint:= Translate((Control as TBevel).Name+'.hint', (Control as TBevel).Hint);
                    end;
                if Control is TScrollBox then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TScrollBox).Hint:= Translate((Control as TScrollBox).Hint);
                    end
                  else
                    begin
                      (Control as TScrollBox).Hint:= Translate((Control as TScrollBox).Name+'.hint', (Control as TScrollBox).Hint);
                    end;
                if Control is TSplitter then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TSplitter).Hint:= Translate((Control as TSplitter).Hint);
                    end
                  else
                    begin
                      (Control as TSplitter).Hint:= Translate((Control as TSplitter).Name+'.hint', (Control as TSplitter).Hint);
                    end;
                if Control is TRichEdit then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TRichEdit).Hint:= Translate((Control as TRichEdit).Hint);
                    end
                  else
                    begin
                      (Control as TRichEdit).Hint:= Translate((Control as TRichEdit).Name+'.hint', (Control as TRichEdit).Hint);
                    end;
                if Control is TTrackBar then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TTrackBar).Hint:= Translate((Control as TTrackBar).Hint);
                    end
                  else
                    begin
                      (Control as TTrackBar).Hint:= Translate((Control as TTrackBar).Name+'.hint', (Control as TTrackBar).Hint);
                    end;}
                if Control is TProgressBar then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TProgressBar).Hint:= Translate((Control as TProgressBar).Hint);
                    end
                  else
                    begin
                      (Control as TProgressBar).Hint:= Translate((Control as TProgressBar).Name+'.hint', (Control as TProgressBar).Hint);
                    end;
                {if Control is TUpDown then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TUpDown).Hint:= Translate((Control as TUpDown).Hint);
                    end
                  else
                    begin
                      (Control as TUpDown).Hint:= Translate((Control as TUpDown).Name+'.hint', (Control as TUpDown).Hint);
                    end;
                if Control is THotKey then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as THotKey).Hint:= Translate((Control as THotKey).Hint);
                    end
                  else
                    begin
                      (Control as THotKey).Hint:= Translate((Control as THotKey).Name+'.hint', (Control as THotKey).Hint);
                    end;
                if Control is TAnimate then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TAnimate).Hint:= Translate((Control as TAnimate).Hint);
                    end
                  else
                    begin
                      (Control as TAnimate).Hint:= Translate((Control as TAnimate).Name+'.hint', (Control as TAnimate).Hint);
                    end;
                if Control is TDateTimePicker then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TDateTimePicker).Hint:= Translate((Control as TDateTimePicker).Hint);
                    end
                  else
                    begin
                      (Control as TDateTimePicker).Hint:= Translate((Control as TDateTimePicker).Name+'.hint', (Control as TDateTimePicker).Hint);
                    end;
                if Control is TMonthCalendar then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TMonthCalendar).Hint:= Translate((Control as TMonthCalendar).Hint);
                    end
                  else
                    begin
                      (Control as TMonthCalendar).Hint:= Translate((Control as TMonthCalendar).Name+'.hint', (Control as TMonthCalendar).Hint);
                    end;}


                // Non-visual components
                if Control is TSaveDialog then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TSaveDialog).Title:= Translate((Control as TSaveDialog).Title);
                      k:= (Control as TSaveDialog).FilterIndex;
                      (Control as TSaveDialog).Filter:= Translate((Control as TSaveDialog).Filter);
                      (Control as TSaveDialog).FilterIndex:= k;
                    end
                  else
                    begin
                      (Control as TSaveDialog).Title:= Translate((Control as TSaveDialog).Name+'.title', (Control as TSaveDialog).Title);
                      k:= (Control as TSaveDialog).FilterIndex;
                      (Control as TSaveDialog).Filter:= Translate((Control as TSaveDialog).Name+'.filter', (Control as TSaveDialog).Filter);
                      (Control as TSaveDialog).FilterIndex:= k;
                    end;
                {if Control is TOpenDialog then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TOpenDialog).Title:= Translate((Control as TOpenDialog).Title);
                      k:= (Control as TOpenDialog).FilterIndex;
                      (Control as TOpenDialog).Filter:= Translate((Control as TOpenDialog).Filter);
                      (Control as TOpenDialog).FilterIndex:= k;
                    end
                  else
                    begin
                      (Control as TOpenDialog).Title:= Translate((Control as TOpenDialog).Name+'.title', (Control as TOpenDialog).Title);
                      k:= (Control as TOpenDialog).FilterIndex;
                      (Control as TOpenDialog).Filter:= Translate((Control as TOpenDialog).Name+'.filter', (Control as TOpenDialog).Filter);
                      (Control as TOpenDialog).FilterIndex:= k;
                    end;}

                // List components
                {if Control is TRadioGroup then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TRadioGroup).Caption:= Translate((Control as TRadioGroup).Caption);
                      (Control as TRadioGroup).Hint:= Translate((Control as TRadioGroup).Hint);
                      for i:= 0 to (Control as TRadioGroup).Items.Count-1 do (Control as TRadioGroup).Items[i]:= Translate((Control as TRadioGroup).Items[i]);
                    end
                  else
                    begin
                      (Control as TRadioGroup).Caption:= Translate((Control as TRadioGroup).Name, (Control as TRadioGroup).Caption);
                      (Control as TRadioGroup).Hint:= Translate((Control as TRadioGroup).Name+'.hint', (Control as TRadioGroup).Hint);
                      for i:= 0 to (Control as TRadioGroup).Items.Count-1 do (Control as TRadioGroup).Items[i]:= Translate((Control as TRadioGroup).Name+'.'+IntToStr(i), (Control as TRadioGroup).Items[i]);
                    end;
                if Control is TListBox then
                  if FTranslateBase = tbCaption then
                    begin
                      k:= (Control as TListBox).ItemIndex;
                      for i:= 0 to (Control as TListBox).Items.Count-1 do (Control as TListBox).Items[i]:= Translate((Control as TListBox).Items[i]);
                      (Control as TListBox).Hint:= Translate((Control as TListBox).Hint);
                      (Control as TListBox).ItemIndex:= k;
                    end
                  else
                    begin
                      k:= (Control as TListBox).ItemIndex;
                      for i:= 0 to (Control as TListBox).Items.Count-1 do (Control as TListBox).Items[i]:= Translate((Control as TListBox).Name+'.'+IntToStr(i), (Control as TListBox).Items[i]);
                      (Control as TListBox).Hint:= Translate((Control as TListBox).Name+'.hint', (Control as TListBox).Hint);
                      (Control as TListBox).ItemIndex:= k;
                    end;}
                if Control is TComboBox then
                  if FTranslateBase = tbCaption then
                    begin
                      k:= (Control as TComboBox).ItemIndex;
                      for i:= 0 to (Control as TComboBox).Items.Count-1 do (Control as TComboBox).Items[i]:= Translate((Control as TComboBox).Items[i]);
  //                    (Control as TComboBox).Text:= Translate((Control as TComboBox).Text);
                      (Control as TComboBox).Hint:= Translate((Control as TComboBox).Hint);
                      (Control as TComboBox).ItemIndex:= k;
                    end
                  else
                    begin
                      k:= (Control as TComboBox).ItemIndex;
                      for i:= 0 to (Control as TComboBox).Items.Count-1 do (Control as TComboBox).Items[i]:= Translate((Control as TComboBox).Name+'.'+IntToStr(i), (Control as TComboBox).Items[i]);
  //                    (Control as TComboBox).Text:= Translate((Control as TComboBox).Name, (Control as TComboBox).Text);
                      (Control as TComboBox).Hint:= Translate((Control as TComboBox).Name+'.hint', (Control as TComboBox).Hint);
                      (Control as TComboBox).ItemIndex:= k;
                    end;
                if Control is TPageControl then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TPageControl).Hint:= Translate((Control as TPageControl).Hint);
                    end
                  else
                    begin
                      (Control as TPageControl).Hint:= Translate((Control as TPageControl).Name+'.hint', (Control as TPageControl).Hint);
                    end;
                if Control is TTabSheet then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TTabSheet).Caption:= Translate((Control as TTabSheet).Caption);
                      (Control as TTabSheet).Hint:= Translate((Control as TTabSheet).Hint);
                    end
                  else
                    begin
                      (Control as TTabSheet).Caption:= Translate((Control as TTabSheet).Name, (Control as TTabSheet).Caption);
                      (Control as TTabSheet).Hint:= Translate((Control as TTabSheet).Name+'.hint', (Control as TTabSheet).Hint);
                    end;
                {if Control is TCheckListBox then
                  if FTranslateBase = tbCaption then
                    begin
                      k:= (Control as TCheckListBox).ItemIndex;
                      for i:= 0 to (Control as TCheckListBox).Items.Count-1 do (Control as TListBox).Items[i]:= Translate((Control as TCheckListBox).Items[i]);
                      (Control as TCheckListBox).Hint:= Translate((Control as TCheckListBox).Hint);
                      (Control as TCheckListBox).ItemIndex:= k;
                    end
                  else
                    begin
                      k:= (Control as TCheckListBox).ItemIndex;
                      for i:= 0 to (Control as TCheckListBox).Items.Count-1 do (Control as TListBox).Items[i]:= Translate((Control as TCheckListBox).Name+'.'+IntToStr(i), (Control as TCheckListBox).Items[i]);
                      (Control as TCheckListBox).Hint:= Translate((Control as TCheckListBox).Name+'.hint', (Control as TCheckListBox).Hint);
                      (Control as TCheckListBox).ItemIndex:= k;
                    end;}
                {if Control is TTabControl then
                  if FTranslateBase = tbCaption then
                    begin
                      for i:= 0 to (Control as TTabControl).Tabs.Count-1 do (Control as TTabControl).Tabs[i]:= Translate((Control as TTabControl).Tabs[i]);
                      (Control as TTabControl).Hint:= Translate((Control as TTabControl).Hint);
                    end
                  else
                    begin
                      for i:= 0 to (Control as TTabControl).Tabs.Count-1 do (Control as TTabControl).Tabs[i]:= Translate((Control as TTabControl).Name+'.'+IntToStr(i), (Control as TTabControl).Tabs[i]);
                      (Control as TTabControl).Hint:= Translate((Control as TTabControl).Name+'.hint', (Control as TTabControl).Hint);
                    end;}
                {if Control is TValueListEditor then
                  if FTranslateBase = tbCaption then
                    begin
                      for i:= 0 to (Control as TValueListEditor).TitleCaptions.Count-1 do
                        (Control as TValueListEditor).TitleCaptions[i]:= Translate((Control as TValueListEditor).TitleCaptions[i]);
                      for i:= 1 to (Control as TValueListEditor).RowCount-1 do
                        begin
                          (Control as TValueListEditor).Values[(Control as TValueListEditor).Keys[i]]:= Translate((Control as TValueListEditor).Values[(Control as TValueListEditor).Keys[i]]);
                          (Control as TValueListEditor).Keys[i]:= Translate((Control as TValueListEditor).Keys[i]);
                        end;
                      (Control as TValueListEditor).Hint:= Translate((Control as TValueListEditor).Hint);
                    end
                  else
                    begin
                      for i:= 0 to (Control as TValueListEditor).TitleCaptions.Count-1 do
                        (Control as TValueListEditor).TitleCaptions[i]:= Translate((Control as TValueListEditor).Name+'.titles.'+IntToStr(i), (Control as TValueListEditor).TitleCaptions[i]);
                      for i:= 1 to (Control as TValueListEditor).RowCount-1 do
                        begin
                          (Control as TValueListEditor).Values[(Control as TValueListEditor).Keys[i]]:= Translate((Control as TValueListEditor).Name+'.values.'+(Control as TValueListEditor).Keys[i], (Control as TValueListEditor).Values[(Control as TValueListEditor).Keys[i]]);
                          (Control as TValueListEditor).Keys[i]:= Translate((Control as TValueListEditor).Name+'.keys.'+IntToStr(i), (Control as TValueListEditor).Keys[i]);
                        end;
                      (Control as TValueListEditor).Hint:= Translate((Control as TValueListEditor).Name+'.hint', (Control as TValueListEditor).Hint);
                    end;}
                {if Control is TTreeView then
                  if FTranslateBase = tbCaption then
                    begin
                      for i:= 0 to (Control as TTreeView).Items.Count-1 do
                        (Control as TTreeView).Items[i].Text:= Translate((Control as TTreeView).Items[i].Text);
                      (Control as TTreeView).Hint:= Translate((Control as TTreeView).Hint);
                    end
                  else
                    begin
                      for i:= 0 to (Control as TTreeView).Items.Count-1 do
                        (Control as TTreeView).Items[i].Text:= Translate((Control as TTreeView).Name+'.'+IntToStr(i), (Control as TTreeView).Items[i].Text);
                      (Control as TTreeView).Hint:= Translate((Control as TTreeView).Name+'.hint', (Control as TTreeView).Hint);
                    end;
                if Control is TListView then
                  if FTranslateBase = tbCaption then
                    begin
                      for i:= 0 to (Control as TListView).Columns.Count-1 do
                        (Control as TListView).Columns[i].Caption:= Translate((Control as TListView).Columns[i].Caption);
                      for i:= 0 to (Control as TListView).Items.Count-1 do
                        begin
                          (Control as TListView).Items[i].Caption:= Translate((Control as TListView).Items[i].Caption);
                          for k:= 0 to (Control as TListView).Items[i].SubItems.Count-1 do
                            (Control as TListView).Items[i].SubItems[k]:= Translate((Control as TListView).Items[i].SubItems[k]);
                        end;
                      (Control as TListView).Hint:= Translate((Control as TListView).Hint);
                    end
                  else
                    begin
                      for i:= 0 to (Control as TListView).Columns.Count-1 do
                        (Control as TListView).Columns[i].Caption:= Translate((Control as TListView).Name+'.columns.'+IntToStr(i), (Control as TListView).Columns[i].Caption);
                      for i:= 0 to (Control as TListView).Items.Count-1 do
                        begin
                          (Control as TListView).Items[i].Caption:= Translate((Control as TListView).Name+'.items.'+IntToStr(i), (Control as TListView).Items[i].Caption);
                          for k:= 0 to (Control as TListView).Items[i].SubItems.Count-1 do
                            (Control as TListView).Items[i].SubItems[k]:= Translate((Control as TListView).Name+'.items.'+IntToStr(i)+'.subitems.'+IntToStr(k), (Control as TListView).Items[i].SubItems[k]);
                        end;
                      (Control as TListView).Hint:= Translate((Control as TListView).Name+'.hint', (Control as TListView).Hint);
                    end;
                if Control is TStatusBar then
                  if FTranslateBase = tbCaption then
                    begin
                      for i:= 0 to (Control as TStatusBar).Panels.Count-1 do
                        (Control as TStatusBar).Panels[i].Text:= Translate((Control as TStatusBar).Panels[i].Text);
                      (Control as TStatusBar).SimpleText:= Translate((Control as TStatusBar).SimpleText);
                      (Control as TStatusBar).Hint:= Translate((Control as TStatusBar).Hint);
                    end
                  else
                    begin
                      for i:= 0 to (Control as TStatusBar).Panels.Count-1 do
                        (Control as TStatusBar).Panels[i].Text:= Translate((Control as TStatusBar).Name+'.'+IntToStr(i), (Control as TStatusBar).Panels[i].Text);
                      (Control as TStatusBar).SimpleText:= Translate((Control as TStatusBar).Name+'.simpletext', (Control as TStatusBar).SimpleText);
                      (Control as TStatusBar).Hint:= Translate((Control as TStatusBar).Name+'.hint', (Control as TStatusBar).Hint);
                    end;
                if Control is TComboBoxEx then
                  if FTranslateBase = tbCaption then
                    begin
  //                    for i:= 0 to (Control as TComboBoxEx).ItemsEx.Count-1 do
  //                      (Control as TComboBoxEx).ItemsEx[i].Caption:= Translate((Control as TComboBoxEx).ItemsEx[i].Caption);
  //                    for i:= 0 to (Control as TComboBoxEx).Items.Count-1 do
  //                      (Control as TComboBoxEx).Items[i]:= Translate((Control as TComboBoxEx).Items[i]);
  //                    (Control as TComboBoxEx).Text:= Translate((Control as TComboBoxEx).Text);
                      (Control as TComboBoxEx).Hint:= Translate((Control as TComboBoxEx).Hint);
                    end
                  else
                    begin
  //                    for i:= 0 to (Control as TComboBoxEx).ItemsEx.Count-1 do
  //                      (Control as TComboBoxEx).ItemsEx[i].Caption:= Translate((Control as TComboBoxEx).Name+'.itemsex.'+IntToStr(i), (Control as TComboBoxEx).ItemsEx[i].Caption);
  //                    for i:= 0 to (Control as TComboBoxEx).Items.Count-1 do
  //                      (Control as TComboBoxEx).Items[i]:= Translate((Control as TComboBoxEx).Name+'.'+IntToStr(i), (Control as TComboBoxEx).Items[i]);
  //                    (Control as TComboBoxEx).Text:= Translate((Control as TComboBoxEx).Name, (Control as TComboBoxEx).Text);
                      (Control as TComboBoxEx).Hint:= Translate((Control as TComboBoxEx).Name+'.hint', (Control as TComboBoxEx).Hint);
                    end;
                if Control is TToolBar then
                  if FTranslateBase = tbCaption then
                    begin
                      for i:= 0 to (Control as TToolBar).ButtonCount-1 do
                        begin
                          (Control as TToolBar).Buttons[i].Caption:= Translate((Control as TToolBar).Buttons[i].Caption);
                          (Control as TToolBar).Buttons[i].Hint:= Translate((Control as TToolBar).Buttons[i].Hint);
                        end;
                      (Control as TToolBar).Hint:= Translate((Control as TToolBar).Hint);
                    end
                  else
                    begin
                      for i:= 0 to (Control as TToolBar).ButtonCount-1 do
                        begin
                          (Control as TToolBar).Buttons[i].Caption:= Translate((Control as TToolBar).Name+'.'+IntToStr(i), (Control as TToolBar).Buttons[i].Caption);
                          (Control as TToolBar).Buttons[i].Hint:= Translate((Control as TToolBar).Name+'.hints.'+IntToStr(i), (Control as TToolBar).Buttons[i].Hint);
                        end;
                      (Control as TToolBar).Hint:= Translate((Control as TToolBar).Name+'.hint', (Control as TToolBar).Hint);
                    end;}
                {if Control is TCheckListBox then
                  if FTranslateBase = tbCaption then
                    begin
                      for i:= 0 to (Control as TCheckListBox).Items.Count-1 do (Control as TCheckListBox).Items[i]:= Translate((Control as TCheckListBox).Items[i]);
                      (Control as TCheckListBox).Hint:= Translate((Control as TCheckListBox).Hint);
                    end
                  else
                    begin
                      for i:= 0 to (Control as TCheckListBox).Items.Count-1 do (Control as TCheckListBox).Items[i]:= Translate((Control as TCheckListBox).Name+'.'+IntToStr(i), (Control as TCheckListBox).Items[i]);
                      (Control as TCheckListBox).Hint:= Translate((Control as TCheckListBox).Name+'.hint', (Control as TCheckListBox).Hint);
                    end;}

                // Group components
                if Control is TForm then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TForm).Caption:= Translate((Control as TForm).Caption);
                      (Control as TForm).Hint:= Translate((Control as TForm).Hint);
                    end
                  else
                    begin
                      (Control as TForm).Caption:= Translate((Control as TForm).Name, (Control as TForm).Caption);
                      (Control as TForm).Hint:= Translate((Control as TForm).Name+'.hint', (Control as TForm).Hint);
                    end;
                if Control is TPanel then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TPanel).Caption:= Translate((Control as TPanel).Caption);
                      (Control as TPanel).Hint:= Translate((Control as TPanel).Hint);
                    end
                  else
                    begin
                      (Control as TPanel).Caption:= Translate((Control as TPanel).Name, (Control as TPanel).Caption);
                      (Control as TPanel).Hint:= Translate((Control as TPanel).Name+'.hint', (Control as TPanel).Hint);
                    end;
                if Control is TGroupBox then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TGroupBox).Caption:= Translate((Control as TGroupBox).Caption);
                      (Control as TGroupBox).Hint:= Translate((Control as TGroupBox).Hint);
                    end
                  else
                    begin
                      (Control as TGroupBox).Caption:= Translate((Control as TGroupBox).Name, (Control as TGroupBox).Caption);
                      (Control as TGroupBox).Hint:= Translate((Control as TGroupBox).Name+'.hint', (Control as TGroupBox).Hint);
                    end;
                {if Control is TTabSheet then
                  if FTranslateBase = tbCaption then
                    begin
                      (Control as TTabSheet).Caption:= Translate((Control as TTabSheet).Caption);
                      (Control as TTabSheet).Hint:= Translate((Control as TTabSheet).Hint);
                    end
                  else
                    begin
                      (Control as TTabSheet).Caption:= Translate((Control as TTabSheet).Name, (Control as TTabSheet).Caption);
                      (Control as TTabSheet).Hint:= Translate((Control as TTabSheet).Name+'.hint', (Control as TTabSheet).Hint);
                    end;}

                // Other components
                {if Control is TChart then
                  if FTranslateBase = tbCaption then
                    begin
                      //
                      (Control as TChart).Hint:= Translate((Control as TChart).Hint);
                    end
                  else
                    begin
                      //
                      (Control as TChart).Hint:= Translate((Control as TChart).Name+'.hint', (Control as TChart).Hint);
                    end;}
              end;
          end;
      end;
end;

procedure TMUI.ProcessParent;
begin
  ProcessForm(FParent);
end;

end.

